@@ -1,5 +1,6 @@
 import logging
 import typing
+import re
 from typing import Any
 
 import pandas as pd
@@ -58,7 +59,12 @@ class SDRFColumn(Column):
         self._optional = optional_type
 
     def validate_optional(self, series):
-        return [error for validation in self.optional_validations for error in validation.get_errors(series, self)]
+        warnings = []
+        for validation in self.optional_validations:
+            for error in validation.get_errors(series, self):
+                w = LogicError(error.message, error.value, error.row, error.column, error_type=logging.WARN)
+                warnings.append(w)
+        return warnings
 
 
 class OntologyTerm(_SeriesValidation):
@@ -121,6 +127,8 @@ class OntologyTerm(_SeriesValidation):
 
 
 class SDRFSchema(Schema):
+    _special_columns = {'sourcename', 'assayname', 'materialtype'}
+    _column_template = r'^(characteristics|comment|factor value)\s*\[([^\]]+)\](?:\.\d+)?$'
 
     def __init__(self, columns: typing.Iterable[SDRFColumn], ordered: bool = False, min_columns: int = 0):
         super().__init__(columns, ordered)
@@ -151,11 +159,30 @@ class SDRFSchema(Schema):
             for error in error_ontology_terms:
                 errors.append(error)
 
-        warnings = self.check_recommendations(panda_sdrf)
-        for w in warnings:
-            logging.warning(w)
+        error_names = self.validate_column_names(panda_sdrf)
+        if error_names:
+            errors.append(error_names)
+
+        errors.extend(self.check_recommendations(panda_sdrf))
+
         return errors
 
+    def validate_column_names(self, panda_sdrf):
+        errors = []
+        spaces = []
+        for cname in panda_sdrf.columns:
+            if cname != cname.strip():
+                spaces.append(cname)
+                continue
+            if cname.replace(' ', '') in self._special_columns:
+                continue
+            m = re.match(self._column_template, cname)
+            if not m:
+                errors.append(cname)
+        if errors + spaces:
+            return LogicError('Invalid columns present: ' + ', '.join(errors) +
+                ', '.join(e + ' (leading or trailing whitespace)' for e in spaces), error_type=logging.ERROR)
+
     def validate_mandatory_columns(self, panda_sdrf):
         error_mandatory = []
         for column in self.columns:
@@ -209,6 +236,9 @@ default_schema = SDRFSchema([
              [LeadingWhitespaceValidation(), TrailingWhitespaceValidation(), OntologyTerm("ncbitaxon")],
              allow_empty=False,
              optional_type=False),
+    SDRFColumn('characteristics[cell type]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
+             allow_empty=False,
+             optional_type=False),
     SDRFColumn('assay name',
              [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
              allow_empty=False,
@@ -272,7 +302,10 @@ plants_chema = SDRFSchema([
 
 
 cell_lines_schema = SDRFSchema([
-    SDRFColumn('characteristics[cell line code]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
+    SDRFColumn('characteristics[cell type]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
+             allow_empty=True,
+             optional_type=False),
+    SDRFColumn('characteristics[cultured cell]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
              allow_empty=True,
              optional_type=False)
 ], min_columns=7)
@@ -298,15 +331,15 @@ mass_spectrometry_schema = SDRFSchema([
     SDRFColumn('comment[modification parameters]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation(),
                                                   OntologyTerm(ontology_name="unimod", not_available=True)],
              allow_empty=True,
-             optional_type=False),
+             optional_type=True),
     SDRFColumn('comment[cleavage agent details]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation(),
                                                  OntologyTerm("ms")],
              allow_empty=True,
              optional_type=False),
     SDRFColumn('comment[fragment mass tolerance]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
              allow_empty=True,
-             optional_type=False),
+             optional_type=True),
     SDRFColumn('comment[precursor mass tolerance]', [LeadingWhitespaceValidation(), TrailingWhitespaceValidation()],
              allow_empty=True,
-             optional_type=False)
+             optional_type=True)
 ], min_columns=7)
